package main

import (
	"encoding/base64"
	"flag"
	"fmt"
	"log"

	"git.autistici.org/ai3/go-common/userenckey"
)

var (
	doGenKeys = flag.Bool("gen-keys", false, "generate user encryption keys with the specified curve")
	doDecrypt = flag.Bool("decrypt", false, "decrypt the private key given on the command line")
	password  = flag.String("password", "", "password")
)

func genKeys() ([]byte, []byte, error) {
	pub, priv, err := userenckey.GenerateKey()
	if err != nil {
		return nil, nil, err
	}

	enc, err := userenckey.Encrypt(priv, []byte(*password))
	if err != nil {
		return nil, nil, err
	}

	return enc, pub, err
}

func main() {
	log.SetFlags(0)
	flag.Parse()

	switch {
	case *doGenKeys:
		if *password == "" {
			log.Fatal("must specify --password")
		}

		priv, pub, err := genKeys()
		if err != nil {
			log.Fatal(err)
		}
		fmt.Printf(
			"public key:\n%s\nprivate key (encrypted,base64): %s\n",
			pub,
			base64.StdEncoding.EncodeToString(priv),
		)

	case *doDecrypt:
		if *password == "" {
			log.Fatal("must specify --password")
		}
		if flag.NArg() < 1 {
			log.Fatal("not enough arguments")
		}

		var encKeys [][]byte
		for _, arg := range flag.Args() {
			encKey, err := base64.StdEncoding.DecodeString(arg)
			if err != nil {
				log.Fatalf("bad base64-encoded argument: %v", err)
			}
			encKeys = append(encKeys, encKey)
		}

		dec, err := userenckey.Decrypt(encKeys, []byte(*password))
		if err != nil {
			log.Fatal(err)
		}
		pem, err := dec.PEM()
		if err != nil {
			log.Fatalf("invalid private key: %v", err)
		}
		fmt.Printf("private key:\n%s\n", pem)

	default:
		log.Fatal("no actions specified")
	}
}
